import _km_omp as _cp
import networkx as nx
from itertools import compress
import numpy as np
import scipy
from scipy.sparse import triu 

def detect(G, ports, resol = 1, phi = {}, num_samples = 100, num_runs=10, consensus_threshold=0.9, significance_level = 0.05, num_rand_nets = 500):
	"""
	Parameters
	----------
	
	G: NetworkX graph
	    Bipartite network composed of N nodes of one type and M nodes of another type.
	    See details in `NetworkX documentation <https://networkx.github.io/documentation/stable/>`_.
	
	ports: list of length N
	    Nodes to project (e.g., specify port nodes to create a network of ports)
	
	resol : float (Optional; Default = 1; 0<=resol)
	    Resolution parameter 
	
	phi : dict of length M (Optional; Default phi[route] = 1 for all routes)
	    - key : route name
	    - value : container capacity 
	
	num_samples: int (Optional; Default = 100; 0 < num_samples)
	    Number of sample CP structures used to obtain consensus CP structure
	
	num_runs: int (Optional; Default = 10; 0 < num_runs)
	    Number of runs of the algorithm to find one sample CP structure
	
	consensus_threshold: float (Optional; Default = 0.9; 0 <= consensus_threshold <=1)
	    Consensus threshold
	
	significance_level: float (Optional; Default = 0.05; 0 < significance_level <=1)
	    Statistical significance level before the Šidák correction
	
	num_rand_nets: int (Optional; Default = 500; 0 < num_rand_nets)
	    Number of randomised networks used to infer the statistical significance
	
	Returns
	-------
	
	c: dict of length N
	    - key: port name
	    - value: index of the consensus CP pair to which the port belongs  
	
	x: dict of length N
	    - key: port name
	    - value: coreness of the port
	"""

	# Make a list of route nodes.  	
	routes = [node for node in G.nodes() if node not in ports]
	
	Np = len(ports) # Number of ports
	Nr = len(routes) # Number of routes
	
	# Convert phi (dict) to numpy array. 
	if len(phi) == 0:
		phi = np.array(np.ones(len(routes))).astype(float)
	else:
		phi = np.array([phi[r] for r in routes]).astype(float)

	# Make the list of edges in the given network
	A = nx.adjacency_matrix(G, ports+routes)
	r, c = triu(A).nonzero()
	edges = np.array([[rc[0], rc[1]] for rc in zip(r, c)]).astype(int)
	
	# Pass parameters to a c++ function (src/_km_ompnet.cpp)	
	results = _cp._detect(edges = edges,\
			ports = np.array(list(range(Np))).astype(int),\
			routes = np.array(list(range(Np, Nr + Np))).astype(int),\
			phi = np.array(phi).astype(float),\
			resol = float(resol),\
			num_samples = int(num_samples),\
			num_runs = int(num_runs),\
			consensus_threshold = float(consensus_threshold),\
			significance_level = float(significance_level),\
			num_rand_nets = int(num_rand_nets))

	# Retrieve the results 
	c = results[0].astype(int)	
	x = results[1]
	
	# Remove the homeless nodes that do not belong to any consensus CP pairs  	
	b = c>=0
	c = dict(zip(compress(ports,b), c[b]))	
	x = dict(zip(compress(ports,b), x[b]))
	
	return c,x	



from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
import sys
import setuptools

import os
from setuptools import setup, find_packages

__version__ = '0.0.4'

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


class get_pybind_include(object):
    """Helper class to determine the pybind11 include path

    The purpose of this class is to postpone importing pybind11
    until it is actually installed, so that the ``get_include()``
    method can be invoked. """

    def __init__(self, user=False):
        self.user = user

    def __str__(self):
        import pybind11
        return pybind11.get_include(self.user)

ext_modules = [
    Extension(
        '_km_omp',
	['src/_km_omp.cpp'],
        include_dirs=[
            # Path to pybind11 headers
            get_pybind_include(),
            get_pybind_include(user=True),
	    'include'
        ],
        language='c++',
        extra_link_args=['-lgomp'],
	extra_compile_args=['-fopenmp']
    ),
]

# As of Python 3.6, CCompiler has a `has_flag` method.
# cf http://bugs.python.org/issue26689
def has_flag(compiler, flagname):
    """Return a boolean indicating whether a flag name is supported on
    the specified compiler.
    """
    import tempfile
    with tempfile.NamedTemporaryFile('w', suffix='.cpp') as f:
        f.write('int main (int argc, char **argv) { return 0; }')
        try:
            compiler.compile([f.name], extra_postargs=[flagname])
        except setuptools.distutils.errors.CompileError:
            return False
    return True


def cpp_flag(compiler):
    """Return the -std=c++[11/14] compiler flag.

    The c++14 is prefered over c++11 (when it is available).
    """
    if has_flag(compiler, '-std=c++14'):
        return '-std=c++14'
    elif has_flag(compiler, '-std=c++11'):
        return '-std=c++11'
    else:
        raise RuntimeError('Unsupported compiler -- at least C++11 support '
                           'is needed!')


class BuildExt(build_ext):
    """A custom build extension for adding compiler-specific options."""
    c_opts = {
        'msvc': ['/EHsc'],
        'unix': [],
    }

    if sys.platform == 'darwin':
        c_opts['unix'] += ['-stdlib=libc++', '-mmacosx-version-min=10.7']

    def build_extensions(self):
        ct = self.compiler.compiler_type
        opts = self.c_opts.get(ct, [])
        if ct == 'unix':
            opts.append('-DVERSION_INFO="%s"' % self.distribution.get_version())
            opts.append(cpp_flag(self.compiler))
            opts.append('-fopenmp')
            if has_flag(self.compiler, '-fvisibility=hidden'):
                opts.append('-fvisibility=hidden')
        elif ct == 'msvc':
            opts.append('/DVERSION_INFO=\\"%s\\"' % self.distribution.get_version())
        for ext in self.extensions:
            ext.extra_compile_args = opts
        build_ext.build_extensions(self)

setup(
    name='multiresolcp',
    version=__version__,
    author='Sadamori Kojaku',
    author_email='freesailing4046@gmail.com',
    description='KM algorithm for networks induced by a one-mode projection of networks',
    long_description="sada",
    url='https://github.com/skojaku/multiresolcp',
    ext_modules=ext_modules,
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['pybind11>=2.2', 'networkx>=2.0', 'numpy>=1.14.2'],
    cmdclass={'build_ext': BuildExt},
    zip_safe=False,
    include_package_data=True,
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='network core-periphery structure bipartite network',
)
